/*
Graph View is placed in the public domain for non-commercial use by not-for-profit educational institutions in the United States.   All other rights reserved.

*/

/*

Add the following files if you want to use a GraphView object in your application:

CommonDefs.h
GraphView.h
libGraphView.a
GraphViewControls.nib
cross.tiff
Unzoom.tiff

*/


#import <appkit/View.h>
#import "CommonDefs.h"

typedef struct
		{
			float redVal;
			float greVal;
			float bluVal;
		}colorType;

typedef struct
		{
			NXDPoint	coords;
			int		penStyle;
			colorType	penColor;
			double		size;
		}storedDataType;
		
typedef struct
		{
			double		edge;
			double		height;
			double		size;
			colorType	penColor;
		}sortedHistogramDataType;

typedef struct
		{
			NXDPoint	data;
			double		binEdge;
			double		size;
			colorType	penColor;
		}rawHistogramDataType;
				
typedef struct
		{	double	xmin, xmax, ymin, ymax;
		}graphExtremeties;
		
#define CENTRE 0
#define TOP 2
#define BOTTOM 1
#define LEFT 1
#define RIGHT 2
#define ORIGIN 3

#define LIFT 0
#define SLINE 1
#define DLINE 2
#define CIRCLE 3
#define SQUARE 4

#define MINIMALASCIICOPY 1
#define FULLASCIICOPY 2

#define	CORRECTMARGINS 0
#define LABELIFMARGIN 1

#define	DPSDATALIMIT 2048
#define	DPSOPLIMIT 1024

#define LINEAR 0
#define LOG 1
#define POLAR 2

#define	HORIZONTAL 0
#define VERTICAL 1

@interface GraphView:View
{	
	id	crossCursor;
	id	unZoomCursor;
	id	dataPoints;
	id 	rawHistPoints;
	id	sortedHistPoints;
	id	xMinField;
	id	xMaxField;
	id	yMinField;
	id	yMaxField;
	id	xMajTickWidthField;
	id	yMajTickWidthField;
	id	xTickStartField;
	id	yTickStartField;
	id	cBinEdgeField;
	id	binWidthField;
	id	binOffsetField;
	id	maxStrokeRatioField;
	id	thePasteBoard;
	id	xAutoSW;
	id	yAutoSW;
	id	nibIncreaseButton;
	id	nibDecreaseButton;
	id	strokeIncreaseButton;
	id	strokeDecreaseButton;
	id	pointRadiusField;
	id	lineThicknessField;
	id	strokeWidthField;
	id	readOut;
	id	zoomList;
	id	delegate;
	id	snapShot;
	id	xPlotModeSelector;
	id	yPlotModeSelector;
	id	oldEdgeField;
	id	newEdgeField;
	id	controlPanel;		//Do not archive!!
	id	histogramInspector;	//Do not archive!!
	
	// Some color definitions
	
	colorType gridColor;
	colorType grBackColor;
	colorType ticksColor;
	colorType scaleColor;
	colorType axesColor;
	colorType titleColor;
	colorType xyTitleColor;
	colorType axesOrderColor;
	colorType marginColor;
	colorType outlineColor;

	// Pen Attributes
	
	colorType	cPenColor;
	int		cPenStyle;
	
	// Miscelleneous
	BOOL    graphViewInitialized;	
	BOOL	xAutoscale;
	BOOL	yAutoscale;
	BOOL	canScroll;
	BOOL	gridVisible;
	BOOL	majTicksVisible;
	BOOL	minTicksVisible;
	BOOL	xAxisVisible;
	BOOL	yAxisVisible;
	BOOL	labelsVisible;
	BOOL	drawingEnabled;
	BOOL	psOpsAllowed;
	BOOL	userWarnedForPSOps;
	BOOL	drawInColor;
	BOOL	snapShotIsCurrent;
	BOOL	snapShotOK;
	BOOL	controlPanelOpen;
	BOOL	axisSynchronized;
	int	histogramOrientation;
	int	xAxisLoc;
	int	yAxisLoc;
	int	numMinorTicks;
	int	maxTickLimit;
	int	normTickLimit;
	int	asciiCopyType;
	int	labelDisplayOption;
	int 	xPlotMode;
	int	yPlotMode;
	double	cBinEdge;		// Required for histograms
	double	binWidth;		// Required for histograms
	double	binOffset;		// Required for histograms
	double	currentNibInc;
	double	autoscaleInc;
	double	xOrder;         	//Order of extremeties
	double	yOrder;
	double	xAxPos, yAxPos, xOLabelPos, yOLabelPos;
	double	xTickStart, yTickStart;
	double	xMajTickWidth;
	double	yMajTickWidth;
	double	lineThickness;
	double	histStrokeWidth;
	double	maxStrokeRatio;		//Fraction of bin width
	double	pointRadius;
	double	xMin;
	double	xMax;
	double	yMin;
	double	yMax;
	double	yMargins;
	double	xMargins;
	double	defLineWidth;
	double	gutterWidth;
	double	xMarginWidth;
	double	yMarginWidth;
	double	titleXPos, titleYPos;
	char    *graphTitlePtr;
	char    *xTitlePtr;
	char    *yTitlePtr;
	
	
	// Line Dashing Patterns
	
	float	pat1[0];
	float	pat2[1];
	float	bbox[4];
}

//	The following procedures initialize the object and control its general behavior;

- initFrame:(const NXRect *)frameRect;
- read:(NXTypedStream*)stream;
- write:(NXTypedStream*)stream;
- readCoordData:(NXTypedStream *)stream;
- readHistData:(NXTypedStream *)stream;
- writeCoordData:(NXTypedStream *)stream;
- writeHistData:(NXTypedStream *)stream;
- awake;

//	These are methods that involve the actual data.

- acceptContentsFrom:(NXDPoint *)anArrayPtr arrayOfLength:(int) aLength asNewTrace: (BOOL) newTrace;
- drawWithContentsFrom:(NXDPoint *)anArrayPtr arrayOfLength:(int) aLength asNewTrace:(BOOL) newTrace;
- clearDataInfo:sender;
- determineXExtremeties;
- determineYExtremeties;
- rescaleX:sender;
- rescaleY:sender;

//	The following methods deal with the scaling of the graphView window.

- setXAutoscale:(BOOL) decision;
- setYAutoscale:(BOOL) decision;
- (BOOL) xAutoscale;
- (BOOL) yAutoscale;
- toggleXAutoscale:sender;
- toggleYAutoscale:sender;
- setAutoscaleIncrementTo:(double) incr;
- (double) autoscaleIncrement;

// GraphView Ranges

- manualXMin:(double) val;
- manualXMax:(double) val;
- manualYMin:(double) val;
- manualYMax:(double) val;
- (double) xMin;
- (double) xMax;
- (double) yMin;
- (double) yMax;
	
- setScrollMode:(BOOL) decision;
- receiveXMinFrom:sender;
- receiveXMaxFrom:sender;
- receiveYMinFrom:sender;
- receiveYMaxFrom:sender;
- setDefaultX;
- setDefaultY;
- postExtremeties;

//	The next few methods deal with the format of the plot window.

- syncAxesToTicks:sender;
- setAxesToOrigin:sender;
- setXAxisLocationTo:(int) loc;
- setYAxisLocationTo:(int) loc;
- setXMajorTickWidthTo:(double) num;
- setYMajorTickWidthTo:(double) num;
- receiveXMajorTickWidthFrom:sender;
- receiveYMajorTickWidthFrom:sender;
- setXTickStartTo:(double) num;
- setYTickStartTo:(double) num;
- receiveXTickStartFrom:sender;
- receiveYTickStartFrom:sender;
- setNumMinorTicksTo:(int) num;
- receiveNumMinorTicksFrom:sender;
- setMaxNumMajTicksTo:(int) num;
- setNormNumMajTicksTo:(int) num;
- (int) xAxisLocation;
- (int) yAxisLocation;
- (double) xMajorTickWidth;
- (double) yMajorTickWidth;
- (int) numMinorTicks;
- (int) maxNumMajTicks;
- (int) normNumMajTicks;

//  Labels and ticks show status

- showMajorTicks:(BOOL) decision;
- showMinorTicks:(BOOL) decision;
- showGrid:(BOOL) decision;
- showXAxis:(BOOL) decision;
- showYAxis:(BOOL) decision;
- showLabels:(BOOL) decision;
- toggleMajorTicks:sender;
- toggleMinorTicks:sender;
- toggleGrid:sender;
- toggleXAxis:sender;
- toggleYAxis:sender;
- toggleLabels:sender;
- showAllTicksAndLabels:(BOOL) decision;
- (BOOL) gridVisible;
- (BOOL) majorTicksVisible;
- (BOOL) minorTicksVisible;
- (BOOL) xAxisVisible;
- (BOOL) yAxisVisible;
- (BOOL) labelsVisible;

// gutters and margins
- setGutterWidthTo:(double)val;
- setXMarginWidthTo:(double)val;
- setYMarginWidthTo:(double)val;
- (double) gutterWidth;
- (double) xMarginWidth;
- (double) yMarginWidth;

// Rendering Options
- setPSOpsAllowed:(BOOL)decision;
- setWarningForPSOps:(BOOL)decision;
- setDrawInColorTo:(BOOL)decision;
- setDrawingEnabledTo:(BOOL)decision;
- (BOOL) psOpsAllowed;
- (BOOL) userWarnedForPSOps;
- (BOOL) drawInColor;
- (BOOL) drawingEnabled;
	
		
//	The following methods concern the plot style and text.

- setTitleTo:(char *) theTitlePtr at:(double)xPercent:(double)yPercent;
- setXTitleTo:(char *) theXTitlePtr;
- setYTitleTo:(char *) theYTitlePtr;
- setLabelDisplayOptionTo:(int) val;
- (int) labelDisplayOption;
- (char *) title;
- (double) titleXPos;
- (double) titleYPos;
- (char *) xTitle;
- (char *) yTitle;

// Modifying pen attributes

- setPointRadiusTo:(double) aSize; 		// As a fraction of the window width in pixels
- setLineThicknessTo:(double) aThickness;	// As a fraction of the window width in pixels
- setNibSizeIncTo:(double) nibInc;
- (double) pointRadius;
- (double) lineThickness;
- (double) currentNibInc;
- changeGlobalPenSize:sender;
- setCurrentPenStyleTo:(int) aStyle;
- (int) currentPenStyle;

// Modifying the histogram
- setHistogramStrokeWidthTo:(double) aWidth;	// As a fraction of the window width in pixels
- changeGlobalHistogramStrokeWidth:sender;
- setMaxStrokeRatioTo:(double) fractionWidth;		// As a fraction of bin width;
- receiveMaxStrokeRatioFrom:sender;
- (double) histogramStrokeWidth;
- (double) maxStrokeRatio;
- relocateEdge:(double)oldEdge to:(double) newEdge;
- relocateEdge:sender;
- setCurrentBinEdgeTo:(double) edge;
- receiveCurrentBinEdgeFrom:sender;
- (double) currentBinEdge;
- setBinWidthTo:(double) width;
- receiveBinWidthFrom:sender;
- (double) binWidth;
- setBinOffsetTo:(double) width;
- receiveBinOffsetFrom:sender;
- (double) binOffset;


// Histogram Orientation

- setHistogramOrientationTo:(int)orientation;
- selectHistogramOrientation:sender;		// To be used with a radio button pair only

// Plotting Modes & associated methods
- setXPlotModeTo:(int) mode;
- selectXPlotModeFrom:sender;
- (int) xPlotMode;
- setYPlotModeTo:(int) mode;
- selectYPlotModeFrom:sender;
- (int) yPlotMode;

//	The following method is used for defining the colors;

- (colorType *) gridColor;
- (colorType *) grBackColor;
- (colorType *) ticksColor;
- (colorType *) scaleColor;
- (colorType *) axesColor;
- (colorType *) titleColor;
- (colorType *) xyTitleColor;
- (colorType *) axesOrderColor;
- (colorType *) marginColor;
- (colorType *) outlineColor;
- (colorType *) currentPenColor;
	
- setTheColorOf:(colorType *)theColor toRGBColor:(float) rVal:(float) gVal:(float) bVal;
- setCurrentPenColorToRGBColor:(float) rVal:(float) gVal:(float) bVal;
- setTheColorOf:(colorType *)theColor toHSBColor:(float) hVal:(float) sVal:(float) bVal;
- setCurrentPenColorToHSBColor:(float) hVal:(float) sVal:(float) bVal;

//	Copying Data

- setAsciiCopyTypeTo:(int)type;
- (int) asciiCopyType;
- copyAsciiDataToPboard:sender;
- copyPSDataToPboard:sender;
- copyArchiveDataToPboard:sender;
- pasteArchiveDataFromPboard:sender;

//	And finally the plotting mathods.

- (BOOL) lockFocusAndClip;
- redrawGraph;
- drawSelf:(const NXRect *)rects :(int)rectCount;
- mouseDown:(NXEvent *)theEvent;
- clearMargins;
- drawAxes:(const NXRect *)rects :(int)rectCount;
- plotTheData:(const NXRect *)rects :(int)rectCount;
- addThePoint:(NXDPoint) aPoint;
- addHistogramPoint:(NXDPoint) anyPoint;
- drawLineFrom:(NXDPoint) firstPoint to:(NXDPoint) secondPoint;
- enableDrawing:sender;
- disableDrawing:sender;
- toggleDrawingState:sender;
- colorOn:sender;
- colorOff:sender;
- toggleColor:sender;

// Inspector for the palette
- (const char *)inspectorName;

// Setting delegate;
- setDelegate:anObject;
- delegate;

// Returning Coordinates
- (NXDPoint) getScreenCoordsFor:(double)xVal:(double)yVal;
- (NXDPoint) getWorldCoordsFor:(double)xVal:(double)yVal;

// Methods to return data
- dataPoints;
- (int) fillDataArray:(NXDPoint *)dataArray ofLength:(int)aLength;
- (int) numberOfPoints;

// Delegate of controlPanel
- windowDidResignKey:sender;
- windowWillClose:sender;

- free;

// Expanding to new window
- expandToNewWindow:sender;

@end
